//Events and story stuff. Cutscenes, intro, tutorial, handling progress, etc.


function EventSystem() {
	this.eventlist = [];
	this.finished_events = [];
	
	/** Trigger an event. */
	this.trigger = function(which) {
		if (this.eventlist[which].started) return; //Don't run this if it's already started
		if (this.isDone(which)) return;
		
		this.finished_events[which] = this.eventlist[which];
		this.eventlist[which].started = true;
		this.eventlist[which].action();
	}
	
	/** Invoke persistence of all of this map's events */
	this.persist = function() {
		if (!IsMapEngineRunning()) return;
		for (var i in this.finished_events) {
			if (this.finished_events[i].map == WORLD.map) this.finished_events[i].persistence();
		}
	}
	
	
	/** Update any non-finished events */
	this.update = function() {
		for (var i in this.eventlist) {
			if (this.eventlist[i].started && !this.eventlist[i].finished && this.eventlist[i].update != undefined)
				this.eventlist[i].update();
		}
	}
	
	/** Check if an event has completed or not */
	this.isDone = function(which) {
		if (this.finished_events[which]) return true;
		else if (this.eventlist[which].finished === true) return true;
		return false;
	}
	
	this.fillEvents();
}


/** Just a function that fills the events "database" */
EventSystem.prototype.fillEvents = function() {
	this.eventlist["key 01 obtained"] = new Event("01",
		function() {
			Speech("Why was my key here?", 120);
			this.persistence();
		}, //event action
		function() { //persistent stuff
			if (DoesPersonExist("item key 01"))
				DestroyPerson("item key 01");
		}
	);
	
	this.eventlist["door 01 opened"] = new Event("01",
		function() { //event action
			Speech("It opened! At last!", 180);
			this.persistence();
		},
		function() { //persistence
			SetTile(18,34,1,0); //No special conditions - destroy this door in both worlds!
			SetTile(18,35,1,0);
			SetTile(18,36,1,0);
		}
	);
	
	
	
	/***********************/
	/** The intro cutscene */
	this.eventlist["intro"] = new Event("01",
		function() {
			MOVEMENT.locked = true; //Freeze, cracka!
			SetPersonVisible("player", false);
			WORLD.music["light"].stop();
			CreatePerson("inbed", "inbed.rss", true);
			SetPersonX("inbed", 24);
			SetPersonY("inbed", 586);
			Speech("Zzz...", 150);
			this.time = 0;
		},
		function() {
		},
		function() { //Update
			this.time++;
			
			//if (DEBUG) { font.drawText(0,290,"Intro event:"+this.time); FlipScreen(); } //Bugs out with screen flashes
			if (ButtonPressed("exit")) { //Skip cutscene
				if (WORLD.state == "light") WORLD.flip();
				MOVEMENT.locked = false;
				MOVEMENT.frozen = false;
				SetPersonVisible("player", true);
				this.time = 1800;
			}
			switch (this.time) {
				case 180: //The old man awakens (how threatening!)
					if (DoesPersonExist("inbed")) DestroyPerson("inbed");
					SetPersonVisible("player", true);
					MOVEMENT.locked = false;
					WORLD.music["light"].play(true);
					Speech("Yawn...", 150);
					FadeText("Arrows/D-pad to move", 300);
					break;
				case 600:
					MOVEMENT.locked = true;
					MOVEMENT.frozen = true;
					SetPersonVisible("player", false);
					CreatePerson("glow", "blobthing.rss", true);
					SetPersonY("glow", GetPersonY("glow")+4);
					CreatePerson("hackerwake", "hackerwake.rss", true);
					SetPersonY("hackerwake", GetPersonY("hackerwake")+5);
					PAUSECAMERA = true;
					SetCameraX(SW/2); SetCameraY(SH+SH/2);
					SetPersonX("player", 50);
					SetPersonY("player", 570);
					Speech("Booting...", 130);
					break;
				case 730:
					Speech("Booting done.", 80);
					break;
				//Then hacker wakes and syncs
				case 980: case 990: //And then put 'im on the floor.
					SetPersonY("hackerwake", GetPersonY("hackerwake")+1);
					break;
				case 1200:
					PAUSECAMERA = false;
					MOVEMENT.locked = false;
					MOVEMENT.frozen = false;
					SetPersonVisible("player", true);
					Speech("There. Almost forgot my dentures...", 120);
					break;
				case 1350:
					FadeText("Z or btn 1 to jump", 300);
					break;
				case 1800:
					Speech("Synchronized. Ready to roll.", 160);
					//Let's finish up
					this.finished = true;
					break;
			}
			//What? Inefficient? Doesn't really matter :P
			if (this.time > 600 && this.time < 1200 && DoesPersonExist("glow")) QueuePersonCommand("glow", COMMAND_ANIMATE, true);
			if (this.time > 820 && this.time < 1000 && DoesPersonExist("hackerwake")) QueuePersonCommand("hackerwake", COMMAND_ANIMATE, true); //Animation length is 180 frames
			if (this.time > 1000 && this.time < 1200) Speech("Synchronizing with world..." + Math.floor(17-(this.time-840)/60), 1);
		}
	);
	
	
	/*********************************/
	/** Hacking the door in level 03 */
	this.eventlist["hacking door in 03"] = new Event("03",
		function() { //action
			if (parseFloat(this.timer/60).toFixed(2) <= 4) { //Not enough time? Cancel hacking!
				this.reset();
				delete this.finished_events["hacking door in 03"];
				Speech("Not enough time to hack atm; wait plz");
				return;
			}
			//Else just continue
		},
		function() { //persistence
			var l = GetPersonLayer("player");
			for (var i = 69; i <= 74; i++) SetTile(74,i,l,5); //Remove door
			SetTile(70,55,l,111);
			SetTile(71,55,l,112);
			SetTile(70,56,l,117);
			SetTile(71,56,l,118);
		},
		function() { //update
			this.time++;
			if (this.time == 1) {
				MOVEMENT.locked = true;
				MOVEMENT.frozen = true;
				SetPersonDirection("player", "east hacker hack");
				Speech("Hacking terminal...", 120);
			}
			if (this.time < 130)
				QueuePersonCommand("player",COMMAND_ANIMATE,true);
			if (this.time == 90)
				Speech("Hacking successful; door opened", 180);
			if (this.time == 131) {
				MOVEMENT.locked = false;
				MOVEMENT.frozen = false;
				this.finished = true;
				this.persistence();
			}
		}
	);
	
	
	
	/*********************************/
	/** Hacking the FLOOR door in 03 */
	// Running out of time, so let's make a huge copied mess
	this.eventlist["hacking floor in 03"] = new Event("03",
		function() { //action
			if (parseFloat(this.timer/60).toFixed(2) <= 4) { //Not enough time? Cancel hacking!
				this.reset();
				delete this.finished_events["hacking floor in 03"];
				Speech("Not enough time to hack atm; wait plz");
				return;
			}
			//Else just continue
		},
		function() { //persistence
			var l = GetPersonLayer("player");
			SetTile(114,46,l,5); //Remove floor
			SetTile(115,46,l,5);
			SetTile(116,46,l,5);
		},
		function() { //update
			this.time++;
			if (this.time == 1) {
				MOVEMENT.locked = true;
				MOVEMENT.frozen = true;
				SetPersonDirection("player", "east hacker hack");
				Speech("Hacking terminal...", 120);
			}
			if (this.time < 130)
				QueuePersonCommand("player",COMMAND_ANIMATE,true);
			if (this.time == 90)
				Speech("Hacking successful; door opened", 180);
			if (this.time == 131) {
				MOVEMENT.locked = false;
				MOVEMENT.frozen = false;
				this.finished = true;
				this.persistence();
			}
		}
	);
	
	
	
	/********************************/
	/** Hacking the LIFT door in 03 */
	this.eventlist["hacking lift in 03"] = new Event("03",
		function() { //action
			if (parseFloat(this.timer/60).toFixed(2) <= 4) { //Not enough time? Cancel hacking!
				this.reset();
				delete this.finished_events["hacking lift in 03"];
				Speech("Not enough time to hack atm; wait plz");
				return;
			}
			//Else just continue
		},
		function() { //persistence
		},
		function() { //update
			this.time++;
			if (this.time == 1) {
				MOVEMENT.locked = true;
				MOVEMENT.frozen = true;
				SetPersonDirection("player", "east hacker hack");
				Speech("Hacking terminal...", 120);
			}
			if (this.time < 130)
				QueuePersonCommand("player",COMMAND_ANIMATE,true);
			if (this.time == 90)
				Speech("Hacking successful; lift activated", 180);
			if (this.time == 131) {
				MOVEMENT.locked = false;
				MOVEMENT.frozen = false;
				this.finished = true;
				this.persistence();
			}
		}
	);
	
}


/** map: what map is this event on?
  * action: what should happen when the event is triggered?
  * persistence: indicate what should be changed persistently in the world (like a disappearing door)
  * update: if defined, have something update for a while.
  */
function Event(map, action, persistence, update) {
	this.time = 0;
	this.started = false;
	this.finished = false;
	
	this.map = map;
	this.action = action;
	this.persistence = persistence;
	this.update = update;
	
	this.reset = function() {
		this.time = 0;
		this.started = false;
		this.finished = false;
	}
}