/*----------------------------------\*
||             input.js             ||
||==================================||
||            Game input.           ||
\*----------------------------------*/
// Note: this is a script I wrote long before Ludum Dare, and is part of my base library.
// I don't want to have to mess around with input too much. :)


/** Control handler that abstracts user controls. Works with both the keyboard and joystick. */
function Controls(type, joynum)
{
  this.type = (type != undefined) ? type : "keyboard";
  this.joynum = joynum ? joynum : 0;
  
  this.changeType = function(p_type)
  {
    this.type = p_type;
    if (p_type == "keyboard")
    {
      this.a = KEY_Z;
      this.b = KEY_X;
      this.x = KEY_S;
      this.y = KEY_A;
      this.l = KEY_Q;
      this.r = KEY_W;
      this.start = KEY_ENTER;
      this.select = KEY_TAB;
      this.left = KEY_LEFT;
      this.right = KEY_RIGHT;
      this.up = KEY_UP;
      this.down = KEY_DOWN;
      this.exit = KEY_ESCAPE;
    }
    
    else if (p_type == "joystick")
    {
      //Directions are not assigned in joystick mode (Sphere uses special functions for these).
      this.a = 0;
      this.b = 1;
      this.x = 2;
      this.y = 3;
      this.start = 4;
      this.select = 5;
      this.l = 6;
      this.r = 7;
      this.exit = -1; //No joystick button defined for exit
    }
  }
  
  //Invoke it immediately (like a constructor).
  this.changeType(this.type);
  
  
  this.pressed = function(key)    //key can be "action", "cancel", "menu".
  {
    if (this.type == "keyboard")
    {
      switch(key)
      {
        case "a": return IsKeyPressed(this.a);
        case "b": return IsKeyPressed(this.b);
        case "x": return IsKeyPressed(this.x);
        case "y": return IsKeyPressed(this.y);
        case "l": return IsKeyPressed(this.l);
        case "r": return IsKeyPressed(this.r);
        case "start": return IsKeyPressed(this.start);
        case "select": return IsKeyPressed(this.select);
        case "exit": return IsKeyPressed(this.exit);
        
        //Extra shortcuts.
        case "confirm": return IsKeyPressed(this.a);
        case "cancel": return IsKeyPressed(this.b);
        case "menu":   return IsKeyPressed(this.start);
        case "run": return IsKeyPressed(this.b);
        
        case "up": return IsKeyPressed(this.up);
        case "down": return IsKeyPressed(this.down);
        case "left": return IsKeyPressed(this.left);
        case "right": return IsKeyPressed(this.right);
        
        //Special cases
        case "any":  //Any button in the scheme at all?
          return this.pressed("a") || this.pressed("b") || this.pressed("x") || this.pressed("y") ||
                 this.pressed("l") || this.pressed("r") || this.pressed("start") || this.pressed("select") ||
                 this.pressed("up") || this.pressed("down") || this.pressed("left") || this.pressed("right");
        
        case "dir":  //Any directional button?
          return this.pressed("up") || this.pressed("down") || this.pressed("left") || this.pressed("right");
        
        //default: return IsAnyKeyPressed();
        default: Abort("ERROR: Button does not exist: " + key);
      }
    }
    
    else if (this.type == "joystick")
    {
      switch(key)
      {
        case "a": return IsJoystickButtonPressed(this.joynum, this.a);
        case "b": return IsJoystickButtonPressed(this.joynum, this.b);
        case "x": return IsJoystickButtonPressed(this.joynum, this.x);
        case "y": return IsJoystickButtonPressed(this.joynum, this.y);
        case "l": return IsJoystickButtonPressed(this.joynum, this.l);
        case "r": return IsJoystickButtonPressed(this.joynum, this.r);
        case "start": return IsJoystickButtonPressed(this.joynum, this.start);
        case "select": return IsJoystickButtonPressed(this.joynum, this.select);
        case "exit": return IsJoystickButtonPressed(this.joynum, this.exit);
        
        //Extra shortcuts.
        case "confirm": return IsJoystickButtonPressed(this.joynum, this.a);
        case "cancel": return IsJoystickButtonPressed(this.joynum, this.b);
        case "menu":   return IsJoystickButtonPressed(this.joynum, this.start);
        case "run": return IsJoystickButtonPressed(this.joynum, this.b);
        
        case "up": return (GetJoystickAxis(this.joynum, JOYSTICK_AXIS_Y) < -0.5);  //0.5 to prevent automatic movement from analog input.
        case "down": return (GetJoystickAxis(this.joynum, JOYSTICK_AXIS_Y) > 0.5); //Now you're required to move the stick at least halfway.
        case "left": return (GetJoystickAxis(this.joynum, JOYSTICK_AXIS_X) < -0.5);
        case "right": return (GetJoystickAxis(this.joynum, JOYSTICK_AXIS_X) > 0.5);
        
        //Special cases
        case "any":  //Any button in the scheme at all?
          return this.pressed("a") || this.pressed("b") || this.pressed("x") || this.pressed("y") ||
                 this.pressed("l") || this.pressed("r") || this.pressed("start") || this.pressed("select") ||
                 this.pressed("up") || this.pressed("down") || this.pressed("left") || this.pressed("right");
        
        case "dir":  //Any directional button?
          return this.pressed("up") || this.pressed("down") || this.pressed("left") || this.pressed("right");
        
        default: Abort("ERROR: Button does not exist: " + key);
      }
    }
  }
  
  
  /** Retrieve what key was pressed, if it was within the spectrum of this input method. */
  this.getButton = function()
  {
    if (this.type == "keyboard")
    {
			while (AreKeysLeft()) //BROKEN, WHY?
			{
				switch(GetKey())
				{
					case this.up: return "up";
					case this.down: return "down";
					case this.left: return "left";
					case this.right: return "right";
					case this.a: return "a";
					case this.b: return "b";
					case this.x: return "x";
					case this.y: return "y";
					case this.l: return "l";
					case this.r: return "r";
					case this.start: return "start";
					case this.select: return "select";
					case this.select: return "exit";
				}
			}
    }
    
    else if (this.type == "joystick")
    {
			if (GetJoystickAxis(this.joynum, JOYSTICK_AXIS_Y) < -0.5) return "up";
			else if (GetJoystickAxis(this.joynum, JOYSTICK_AXIS_Y) > 0.5) return "down";
			else if (GetJoystickAxis(this.joynum, JOYSTICK_AXIS_X) < -0.5) return "left";
			else if (GetJoystickAxis(this.joynum, JOYSTICK_AXIS_X) > 0.5) return "right";
      else if (IsJoystickButtonPressed(this.joynum, this.a)) return "a";
      else if (IsJoystickButtonPressed(this.joynum, this.b)) return "b";
      else if (IsJoystickButtonPressed(this.joynum, this.x)) return "x";
      else if (IsJoystickButtonPressed(this.joynum, this.y)) return "y";
      else if (IsJoystickButtonPressed(this.joynum, this.l)) return "l";
      else if (IsJoystickButtonPressed(this.joynum, this.r)) return "r";
      else if (IsJoystickButtonPressed(this.joynum, this.start)) return "start";
      else if (IsJoystickButtonPressed(this.joynum, this.select)) return "select";
      else if (IsJoystickButtonPressed(this.joynum, this.exit)) return "exit";
    }
  }
}
