//Enjoy my messy, bad code

RequireSystemScript("colors.js");
RequireScript("input.js"); //Written before Ludum Dare; part of base library stuff
RequireScript("movement.js");
RequireScript("effects.js");
RequireScript("inventory.js");
RequireScript("camera.js");
RequireScript("events.js");
RequireScript("world.js");

//Basic resources
var font = LoadFont("minifonty.rfn");

//Global variables (expecting "Gack, how terrible" comments)
var DEBUG = false;
var FPS = 60;
var SW = GetScreenWidth();
var SH = GetScreenHeight();
var KEYBOARD = new Controls("keyboard");
var JOYSTICK = new Controls("joystick");

var MOVEMENT;
var WORLD;
var INVENTORY;
var STATIC = 0; //Values 0-255 will show static in that degree
var EFFECTS = [];
var EVENTS;


/*  ==================================================================================== */
/** This is where Sphere always starts running the game (after running the global stuff) */
function game() {
	TitleScreen();
	
	BindKey(KEY_ESCAPE, "", ""); //Prevent default quit with escape
	if (DEBUG) {
		BindKey(KEY_K, "MOVEMENT.die();", "");
		BindKey(KEY_E, "EndGame();", "");
	}
	
	CreatePerson("player", "player.rss", false);
	MOVEMENT = new PlayerMovement();
	INVENTORY = new Inventory();
	EVENTS = new EventSystem();
	WORLD = new World();
	WORLD.init(); //Game code continues in world.js
}



/** Woo! */
function TitleScreen() {
	var titleimg = LoadImage("title.png");
	var text = "PRESS KEY OR BUTTON";
	var x = (SW-font.getStringWidth(text))/2;
	var y = SH-40;
	var timer = 0;
	var blink = true;
	
	SetFrameRate(60);
	while (!ButtonPressed("a") && !ButtonPressed("b") && !ButtonPressed("start")) {
		if (ButtonPressed("exit")) Exit();
		timer++;
		if (timer == 40) { timer = 0; blink = !blink; }
		
		titleimg.blit(0, 0);
		if (blink) font.drawText(x, y, text);
		FlipScreen();
	}
	SetFrameRate(0); //To prevent a certain bug regarding SetFrameRate in Sphere
} 



/** Runs each frame, takes care of rendering stuff on top of the map */
function RenderScript() {
	WORLD.render();
	for (var i in EFFECTS) EFFECTS[i].render();
	INVENTORY.render();
	
	if (DEBUG) {
		font.setColorMask(White);
		font.drawText(0, 0, "camy:" + cameray);
		MOVEMENT.debug();
		WORLD.debug();
	}
}


/** Runs each frame (prior to rendering), takes care of game logic */
function UpdateScript() {
	UpdateCamera();
	WORLD.update();
	MOVEMENT.update();
	INVENTORY.update();
	EVENTS.update();
}



/* Convenience function for catching both keyboard and gamepad input */
function ButtonPressed(which) {
	return KEYBOARD.pressed(which) || JOYSTICK.pressed(which);
}



function EndGame() {
	WORLD.music["light"].stop();
	WORLD.music["dark"].stop();
	var sound = LoadSound("lift.ogg");
	sound.play(false);
	MOVEMENT.deathcount = 8;
	
	while(!ButtonPressed("a")) {
		font.drawTextBox(20, 20, SW-40, SH, 0,
			"And so, the old man stepped into the elevator...\n\n"+
			"He had no clue where it would go, but then again, he did not know how he got here to begin with. "+
			"In the past minutes, despite seemingly having traveled a long distance, he had no recollection of most of the trip. He only remembered constantly appearing in new places...\n\n"+
			((MOVEMENT.deathcount <= 1) ?
				"The old man felt as if he had successfully averted many a life-threatening situation in the past few minutes. What could it mean?" :
				"Not only that, but the man feels as if he had been experiencing a series of deaths... Surely that can't be possible... Right?"
			) + "\n\n" +
			"In any case...\n\n"+
			"He pressed the button, and the lift started moving.\n\n"+
			"Along with the man who hacked his very life.\n\n\n"+
			"The End\nPress Z or button 1"
		);
		FlipScreen();
	}
	
	while(ButtonPressed("any")) { while (AreKeysLeft()) GetKey() }; //Wait for keystuff
	while(!ButtonPressed("a")) {
		font.drawTextBox(20, 20, SW-40, SH, 0,
			"Credits\n\n"+
			"Game made by Vincent Beers (aka DaVince) for the Ludum Dare compo 27. That includes all the graphics, code, music, fonts etcetera.\n\n" +
			"Made with Sphere, a GPL2 open-source 2D game and RPG engine.\nhttp://spheredev.org/\n\n"+
			"Many thanks to the Spherical website and community for always being there for me (the documentation and people are invaluable)! Go check out Radnen and Flying Jester, they're two community members who also joined LD27!\n\n"+
			"All resources and code are available in the folder games/FlippingMatrix. They're all licensed under Creative Commons Unported Attribution 3.0. That means feel free to take or edit anything as long as you credit me! :)\n\n"+
			"Press Z or button 1"
		);
		FlipScreen();
	}
	
	while(ButtonPressed("any")) { while (AreKeysLeft()) GetKey() }; //Wait for keystuff
	while(!ButtonPressed("a")) {
		font.drawTextBox(20, 20, SW-40, SH, 0,
			"Afterword\n\n"+
			"Man... This game turned out a lot shorter than I had hoped! On the second day, time kinda crept up on me, heh... So many features untouched (hardly any puzzles?)! So few maps! Ah well. It's nobody's fault but mine. :]\n\n"+
			"So, about this game. As you could probably tell, I went with the concept of flipping between two realities. Basically, the name \"Flippin' Matrix\" is based on how this is supposed to be a world much like the Matrix, except that this time a real-world hacker type person hacks into another person's reality, possessing him... and kind of failing, too! He can only stay synced for about 10 seconds, making the poor old man experience some insanity.\n\n"+
			"Press Z or button 1"
		);
		FlipScreen();
	}
	
	while(ButtonPressed("any")) { while (AreKeysLeft()) GetKey() }; //Wait for keystuff
	while(!ButtonPressed("a")) {
		font.drawTextBox(20, 20, SW-40, SH, 0,
			"Afterword (2)\n\n"+
			"This was my first \"Let 'em dare\" (shut up, I /like/ pronouncing it wrong! :P) and man was it a challenge. I've seen some other great concepts fly by on the blog and I'm looking forward to playing as many as I can!\n\n"+
			"So... That's it for the notes! Thanks for playing! Hope you enjoyed what was there. :)\n\n\n"+
			"DaVince"
		);
		FlipScreen();
	}
	while(ButtonPressed("any")) { while (AreKeysLeft()) GetKey() }; //Wait for keystuff
	Exit();
}